package com.njyunzhi.pet_identity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.njyunzhi.pet_identity.common.BaseController;
import com.njyunzhi.pet_identity.common.Constants;
import com.njyunzhi.pet_identity.common.ResponseBean;
import com.njyunzhi.pet_identity.common.StringUtils;
import com.njyunzhi.pet_identity.entity.SysUser;
import com.njyunzhi.pet_identity.entity.TaPerson;
import com.njyunzhi.pet_identity.vo.AuditParam;
import com.njyunzhi.pet_identity.vo.MakeCardParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.njyunzhi.pet_identity.service.ITaApplicationService;
import com.njyunzhi.pet_identity.entity.TaApplication;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * <p>
    * 我的申请 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-05-24
 */

@Api(tags = "我的申请")
@RestController
@RequestMapping("/")
public class TaApplicationController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaApplicationController.class);

    @Autowired
    public ITaApplicationService iTaApplicationService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/application",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taApplicationList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

        SysUser sysUser = currentUser();

        IPage<TaApplication> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaApplication> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETE);

        // 审核人角色
        if (Constants.USER_AUDITOR.equals(sysUser.getRoleName())) {
            queryWrapper.ge("status", Constants.WORKFLOW_STATUS_PROCESSING);
            queryWrapper.le("status", Constants.WORKFLOW_STATUS_AUDIT);
        }

        // 发证(制卡)人角色
        if (Constants.USER_MAKER.equals(sysUser.getRoleName())) {
            queryWrapper.ge("status", Constants.WORKFLOW_STATUS_AUDIT);
            queryWrapper.le("status", Constants.WORKFLOW_STATUS_MADE);
        }

        queryWrapper.orderByAsc("status");
        queryWrapper.orderByDesc("create_date");

        IPage<TaApplication> result = iTaApplicationService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/wx/{client}/application",method= RequestMethod.GET)
    @ApiOperation(value="微信列表", notes = "微信列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean wxList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                               @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

        TaPerson taPerson = currentPerson();

        IPage<TaApplication> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaApplication> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.eq("person_id", taPerson.getPersonId());

        queryWrapper.orderByDesc("create_date");

        IPage<TaApplication> result = iTaApplicationService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taApplication 实体对象
     * @return
     */
    @RequestMapping(value="/wx/{clientId}/application",method= RequestMethod.POST)
    @ApiOperation(value="微信申请", notes = "微信申请", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taApplicationAdd(@ApiParam("保存内容") @RequestBody TaApplication taApplication) throws Exception{

        TaPerson taPerson = currentPerson();
        if (StringUtils.isEmpty(taPerson.getPhone())) {
            return ResponseBean.error("请先完善个人信息");
        }

        taApplication.setPersonId(taPerson.getPersonId());
        taApplication.setPersonName(taPerson.getNickName());
        taApplication.setPhone(taPerson.getPhone());
        taApplication.setPayStatus(Constants.PAY_STATUS_READY);
        taApplication.setStatus(Constants.WORKFLOW_STATUS_READY);
        taApplication.setCreateDate(LocalDateTime.now());
        taApplication.setVerifyStatus(Constants.AUDIT_STATUS_READY);
        taApplication.setMakeStatus(Constants.MAKE_STATUS_READY);

        if (iTaApplicationService.save(taApplication)){
            return ResponseBean.success(taApplication);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/wx/{clientId}/application/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="微信删除", notes = "微信删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taApplicationDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        TaPerson taPerson = currentPerson();

        TaApplication taApplication = iTaApplicationService.getById(id);
        if (null == taApplication || taApplication.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到数据");
        }

        if (!taPerson.getPersonId().equals(taApplication.getPersonId())) {
            return ResponseBean.error("无权操作");
        }

        if (taApplication.getStatus() >= Constants.WORKFLOW_STATUS_PROCESSING) {
            return ResponseBean.error("流程已提审, 不能删除");
        }

        if(iTaApplicationService.removeLogicById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taApplication 实体对象
     * @return
     */
    @RequestMapping(value="/admin/application/{id}/audit",method= RequestMethod.PUT)
    @ApiOperation(value="审批", notes = "审批", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean audit(@ApiParam("对象ID") @PathVariable Integer id,
                                        @ApiParam("更新内容") @RequestBody AuditParam auditParam) throws Exception{

        SysUser sysUser = currentUser();
        if (!checkRole(sysUser.getRoleName(), Constants.USER_AUDITOR)) {
            return ResponseBean.error("暂无权限");
        }

        TaApplication taApplication = iTaApplicationService.getById(id);
        if (null == taApplication || taApplication.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到数据");
        }

        if (taApplication.getStatus() < Constants.WORKFLOW_STATUS_PROCESSING) {
            return ResponseBean.error("用户未支付");
        }
        if (taApplication.getStatus() >= Constants.WORKFLOW_STATUS_AUDIT) {
            return ResponseBean.error("当前记录已审批");
        }

        if (auditParam.getVerifyStatus() == Constants.AUDIT_STATUS_REJECT && StringUtils.isEmpty(auditParam.getRejectReason())) {
            return ResponseBean.error("请填写驳回理由");
        }

        taApplication.setVerifyStatus(auditParam.getVerifyStatus());
        taApplication.setVerifyDate(LocalDateTime.now());
        taApplication.setVerifyUser(sysUser.getUserId());
        taApplication.setVerifyUserName(sysUser.getUserName());

        if (iTaApplicationService.updateById(taApplication)){
            return ResponseBean.success(iTaApplicationService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taApplication 实体对象
     * @return
     */
    @RequestMapping(value="/admin/application/{id}/make",method= RequestMethod.PUT)
    @ApiOperation(value="发证", notes = "发证", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean makeCard(@ApiParam("对象ID") @PathVariable Integer id,
                              @ApiParam("更新内容") @RequestBody MakeCardParam makeCardParam) throws Exception{
        SysUser sysUser = currentUser();
        if (!checkRole(sysUser.getRoleName(), Constants.USER_MAKER)) {
            return ResponseBean.error("暂无权限");
        }

        TaApplication taApplication = iTaApplicationService.getById(id);
        if (null == taApplication || taApplication.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到数据");
        }

        if (taApplication.getStatus() != Constants.WORKFLOW_STATUS_AUDIT && taApplication.getVerifyStatus() == Constants.AUDIT_STATUS_PASSED) {
            return ResponseBean.error("申请未通过或状态不对");
        }

        if (taApplication.getApplyMethod() == Constants.APPLY_METHOD_EXPRESS) {
            if (StringUtils.isEmpty(taApplication.getTrackingType()) || StringUtils.isEmpty(taApplication.getTrackingNo())) {
                return ResponseBean.error("请填写快递信息");
            }
        }

        taApplication.setMakeStatus(Constants.MAKE_STATUS_MADE);
        taApplication.setMakeDate(LocalDateTime.now());
        taApplication.setMakeUser(sysUser.getUserId());
        taApplication.setMakeUserName(sysUser.getUserName());
        taApplication.setTrackingType(makeCardParam.getTrackingType());
        taApplication.setTrackingNo(makeCardParam.getTrackingNo());

        if (iTaApplicationService.updateById(taApplication)){
            return ResponseBean.success(iTaApplicationService.getById(id));
        } else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/application/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taApplicationGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{

        TaApplication taApplication = iTaApplicationService.getById(id);
        if (taApplication == null || taApplication.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到数据");
        }

        return ResponseBean.success(taApplication);
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/wx/{clientId}/application/{id}",method= RequestMethod.GET)
    @ApiOperation(value="微信详情", notes = "微信详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean wxGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{

        TaApplication taApplication = iTaApplicationService.getById(id);
        if (taApplication == null || taApplication.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到数据");
        }

        TaPerson taPerson = currentPerson();
        if (!taPerson.getPersonId().equals(taApplication.getPersonId())) {
            return ResponseBean.error("未找到数据");
        }

        return ResponseBean.success(taApplication);
    }
}
